#!/usr/bin/perl -Tw
# autodns 0.0.3
# Copyright 1999-2000 Project Purple. Written by Jonathan McDowell
# http://www.earth.li/projectpurple/progs/autodns.html
# Released under the GPL.

use strict;
use IPC::Open3;

$ENV{'PATH'}="/usr/local/bin:/usr/bin:/bin:/usr/sbin";

my ($from, $subject, $gpguser, $gpggood);
my ($user, $server, $inprocess, $count);
my ($domain, @MAIL, @GPGERROR, @COMMANDS, @zones, %zones, $cfgfile);
my ($me, $ccreply, $conffile, $domainlistroot, @cfgfiles, $VERSION);

$VERSION="0.0.3";
$conffile="/home/noodles/blackcat/secondary.conf";
$domainlistroot="/home/noodles/blackcat/domains.";
@cfgfiles=("/etc/named.conf", "/var/named/named.secondary.conf");
$ccreply="noodles\@earth.li";
$me="autodns\@earth.li";

$count=$inprocess=0;
$gpggood=1;

# Read in the mail from stdin.
@MAIL=<>;

# Now lets try to find out who it's from.
foreach (@MAIL) {
	if (/^$/) { last; }
	if (/^From: (.*)/) { $from=$1; chomp $from;}
	if (/^Subject: (.*)/) { $subject=$1; chomp $subject;}
}

if ((! defined($from)) || $from =~ /^$/ ) {
	die "Couldn't find a from address.";
} elsif ($from =~ /mailer-daemon@/i) {
	die "From address is mailer-daemon, ignoring.";
}

if (! defined($subject)) { $subject="Reply from AutoDNS"; };

# We've got a from address. Start a reply.

open(REPLY, "|sendmail -t -oem -oi") or die "Couldn't spawn sendmail";

print REPLY "From: $me
To: $from
Cc: $ccreply
Subject: Re: $subject

AutoDNS $VERSION
Copyright 1999 Project Purple. Written by Jonathan McDowell
Released under the GPL.

";

# Now run GPG against our incoming mail.
open3(\*GPGIN, \*GPGOUT, \*GPGERR, "gpg --batch");

# Feed it the mail.
print GPGIN @MAIL;
close GPGIN;

# And grab what it has to say.
@GPGERROR=<GPGERR>;
@COMMANDS=<GPGOUT>;
close GPGERR;
close GPGOUT;

# Check who it's from and if the signature was a good one.
foreach (@GPGERROR) {
	chomp;
	if (/Signature made.* (.*)$/) { $gpguser=$1; }
	if (/BAD signature/) { $gpggood = 0; }
}

# If we have an empty user it probably wasn't signed.
if ($gpguser =~ /^$/) {
	print REPLY "Message appears not to be GPG signed.\n";
	close REPLY;
	exit;
}

# Check the signature we got was ok.
if ($gpggood) {
	print REPLY "Good GPG signature found. ($gpguser)\n";
} else {
	print REPLY "Bad GPG signature!\n";
	close REPLY;
	exit;
}

# Now let's check if we know this person.
# Setup your GPG key IDs here.
if ($gpguser =~ /5B430367/) {
	$user = "noodles";
	$server = "193.82.129.227";
}

if ($user =~ /^$/ ) {
	print REPLY "Unknown user.\n";
	close REPLY;
	exit;
}

print REPLY "Got user '$user'\n";

# Ok, now we should figure out what domains we already know about.
undef %zones;
foreach $cfgfile (@cfgfiles) {
	getzones($cfgfile);
}

foreach (@COMMANDS) {
	s/\r//;
	if (/^END$/) {
		$inprocess=0;
	}
	if ($inprocess) {
		print REPLY ">>>$_";
	}
	if (/^BEGIN$/) {
		$inprocess=1;
	}
	if ($inprocess && /^ADD (.*)$/) {
		$domain = $1;
		chomp $domain;

		# Convert domain to lower case.
		$domain =~ tr/[A-Z]/[a-z]/;
		if ($domain =~ /[^-a-z0-9.]/) {
			$domain =~ s/[-a-z0-9.]//g;
			print "Invalid character(s) in domain name: $domain\n";
		} else {
			$count++;
			print REPLY "Adding domain $domain\n";
			push @zones,$domain;

			open (DOMAINSFILE, ">>$conffile");
			print DOMAINSFILE "
### Domain added for '$user'

zone \"$domain\" {
	type slave;
	masters { $server; };
	file \"secondary/$user/$domain\";
	allow-transfer { none; };
	allow-query { any; };
};\n";
			close DOMAINSFILE;

			open (DOMAINLIST, ">>$domainlistroot$user") or die "Couldn't open file.\n";
			print DOMAINLIST "$domain\n";
			close DOMAINLIST;
		}
	}
	if ($inprocess && /^LIST$/) {
		print REPLY "Listing domains for user $user\n";
		print REPLY "------\n";
		open (DOMAINLIST, "<$domainlistroot$user");
		while (<DOMAINLIST>) {
			print REPLY;
		}
		close (DOMAINLIST);
		print REPLY "------\n";
	}
}

print REPLY "Added $count domains\n";
if ($count>0) {
	system("sudo", "ndc", "reload");
}
close REPLY;

exit 0;

### Try to figure out what zones we currently know about.
sub getzones {
	my ($namedfile) = @_;

	open (NAMEDCONF, "< $namedfile") or die "Can't open $namedfile";

	while (<NAMEDCONF>) {
		if (/^ *zone *"(.*)"/) {
			printf "Got domain $1\n";
			$zones{$1}=1;
		}
	}

	close NAMEDCONF;
}
